package cn.wisenergy.web.shir.realm;

import cn.wisenergy.common.utils.Md5Util;
import cn.wisenergy.mapper.AccountMapper;
import cn.wisenergy.mapper.StaffMapper;
import cn.wisenergy.mapper.UsersMapper;
import cn.wisenergy.model.app.AccountInfo;
import cn.wisenergy.model.app.Staff;
import cn.wisenergy.model.app.User;
import cn.wisenergy.model.enums.SceneType;
import cn.wisenergy.model.enums.SourceType;
import cn.wisenergy.service.cache.RedisService;
import cn.wisenergy.service.common.CachePrefix;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.shiro.authc.*;
import org.apache.shiro.authz.AuthorizationInfo;
import org.apache.shiro.realm.AuthorizingRealm;
import org.apache.shiro.session.Session;
import org.apache.shiro.subject.PrincipalCollection;
import org.apache.shiro.util.ByteSource;
import org.springframework.beans.factory.annotation.Autowired;

public class Realm extends AuthorizingRealm {
    @Autowired
    private UsersMapper usersMapper;
    @Autowired
    private AccountMapper accountMapper;
    @Autowired
    private StaffMapper staffMapper;
    @Autowired
    private RedisService redisService;

    /**
     * 执行授权逻辑
     */
    @Override
    protected AuthorizationInfo doGetAuthorizationInfo(PrincipalCollection principal) {
        return null;
    }

    /**
     * 执行认证逻辑
     */
    @Override
    protected AuthenticationInfo doGetAuthenticationInfo(AuthenticationToken token) throws AuthenticationException {

        UsernamePasswordToken tok = (UsernamePasswordToken) token;
        //管理端登录
        if (tok.getHost().equals("admin")) {
            QueryWrapper<AccountInfo> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("user_name", tok.getUsername());
            queryWrapper.eq("is_delete", 0);
            AccountInfo accountInfo = accountMapper.selectOne(queryWrapper);
            if (accountInfo == null) {
                return null;
            }
            String password = Md5Util.digestMD5(accountInfo.getPassword());
            SimpleAuthenticationInfo authenticationInfo = new SimpleAuthenticationInfo(
                    accountInfo,
                    password,
                    accountInfo.getUserName()
            );

            return authenticationInfo;
        }
        //密码登录
        else if (tok.getHost().equals("user")) {
            //通过username从数据库中查找 User对象
            QueryWrapper<User> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("phone", tok.getUsername());
            queryWrapper.eq("is_delete", 0);
            User userInfo = usersMapper.selectOne(queryWrapper);
            if (userInfo == null) {
                return null;
            }
            SimpleAuthenticationInfo authenticationInfo = new SimpleAuthenticationInfo(
                    userInfo,
                    userInfo.getPassword(),
                    userInfo.getPhone()
            );
            return authenticationInfo;
        }
        //短信验证码登录
        else if (tok.getHost().equals("code")) {
            QueryWrapper<User> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("phone", tok.getUsername());
            queryWrapper.eq("is_delete", 0);
            User userInfo = usersMapper.selectOne(queryWrapper);
            if (userInfo == null) {
                return null;
            }
            String source = SourceType.getByCode(userInfo.getSource());
            //获取短信验证码key
            String key = CachePrefix.SMS_CODE.getPrefix()  + source + "_" + userInfo.getPhone();
            String code = redisService.get(key).toString();
            SimpleAuthenticationInfo authenticationInfo = new SimpleAuthenticationInfo(
                    userInfo,
                    code,
                    userInfo.getPhone()
            );
            return authenticationInfo;

        }
        //员工端登录
        else {
            QueryWrapper<Staff> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("login_name", tok.getUsername());
            queryWrapper.eq("is_delete", 0);
            Staff staff = staffMapper.selectOne(queryWrapper);
            if (staff == null) {
                return null;
            }
            SimpleAuthenticationInfo authenticationInfo = new SimpleAuthenticationInfo(
                    staff,
                    staff.getPassword(),
                    staff.getLoginName()
            );
            return authenticationInfo;
        }
    }
}
